module AUDIO_DAC (
					//	Audio Side
					oAUD_BCK,
					oAUD_DATA,
					oAUD_LRCK,
					//	Control Signals
					iSrc_Select,
				    iCLK_18_4,
					iRST_N,	
					TONE1_SET,
					TONE2_SET,
					TONE3_SET,
					NOISE_SET,
					VOL1,
					VOL2,
					VOL3,
					VOL4
					);				

	parameter	REF_CLK			=	12000000;	//	18.432	MHz
	parameter	SAMPLE_RATE		=	46875;		//	48		KHz
	parameter	DATA_WIDTH		=	16;			//	16		Bits
	parameter	CHANNEL_NUM		=	2;				//	Dual Channel

	output	oAUD_DATA;
	output	oAUD_LRCK;
	output	reg	oAUD_BCK;

	input		iSrc_Select;
	input			iCLK_18_4;
	input			iRST_N;
	input [9:0] TONE1_SET;
	input [9:0] TONE2_SET;
	input [9:0] TONE3_SET;
	input [2:0] NOISE_SET;

	input [3:0] VOL1;
	input [3:0] VOL2;
	input [3:0] VOL3;
	input [3:0] VOL4;

	reg		[3:0]	BCK_DIV;
	reg		[8:0]	LRCK_1X_DIV;
	reg		[3:0]	SEL_Cont;

	reg		[15:0] Sample_Out;
	reg		LRCK_1X;

	reg		[9:0] FREQ1_CONT;
	reg		[9:0] FREQ2_CONT;
	reg		[9:0] FREQ3_CONT;
	reg		[9:0] FREQ4_CONT;
	reg		CH1_S;
	reg		CH2_S;
	reg		CH3_S;
	reg		CH4_S;
	reg		CH4_C;
	wire		[15:0] VOL1_LOG;
	wire		[15:0] VOL2_LOG;
	wire		[15:0] VOL3_LOG;
	wire		[15:0] VOL4_LOG;

	reg		[15:0] LFSR;

	////////////	AUD_BCK Generator	//////////////
	always@(posedge iCLK_18_4 or negedge iRST_N)
	begin
		if(!iRST_N)
		begin
			BCK_DIV		<=	0;
			oAUD_BCK	<=	0;
		end
		else
		begin
			if(BCK_DIV >= REF_CLK/(SAMPLE_RATE*DATA_WIDTH*CHANNEL_NUM*2)-1 )
			begin
				BCK_DIV		<=	0;
				oAUD_BCK	<=	~oAUD_BCK;
			end
			else
			BCK_DIV		<=	BCK_DIV+1;
		end
	end

	//////////////////////////////////////////////////
	////////////	AUD_LRCK Generator	//////////////
	always@(posedge iCLK_18_4 or negedge iRST_N)
	begin
		if(!iRST_N)
		begin
			LRCK_1X_DIV	<=	0;
			LRCK_1X <= 0;
		end
		else
		begin
			//	LRCK 1X
			if(LRCK_1X_DIV >= REF_CLK/(SAMPLE_RATE*2)-1 )
			begin
				LRCK_1X_DIV	<=	0;
				LRCK_1X <= ~LRCK_1X;
			end
			else
			LRCK_1X_DIV	<=	LRCK_1X_DIV+1;
		end
	end
	assign	oAUD_LRCK =	LRCK_1X;

	VOL_LUT  v1(VOL1,VOL1_LOG),
				v2(VOL2,VOL2_LOG),
				v3(VOL3,VOL3_LOG),
				v4(VOL4,VOL4_LOG);

	always@(negedge LRCK_1X)
	begin
		if (FREQ1_CONT == 0)
		begin
			FREQ1_CONT <= TONE1_SET[9:1];
			CH1_S <= ~CH1_S;
		end
		else
		begin
			FREQ1_CONT <= FREQ1_CONT - 1;
		end
		
		if (FREQ2_CONT == 0)
		begin
			FREQ2_CONT <= TONE2_SET[9:1];
			CH2_S <= ~CH2_S;
		end
		else
		begin
			FREQ2_CONT <= FREQ2_CONT - 1;
		end
		
		if (FREQ3_CONT == 0)
		begin
			FREQ3_CONT <= TONE3_SET[9:1];
			CH3_S <= ~CH3_S;
		end
		else
		begin
			FREQ3_CONT <= FREQ3_CONT - 1;
		end
		
		if (FREQ4_CONT == 0)
		begin
			case(NOISE_SET[1:0])
			2'b00 : FREQ4_CONT <= 16>>1;
			2'b01 : FREQ4_CONT <= 32>>1;
			2'b10 : FREQ4_CONT <= 64>>1;
			2'b11 : FREQ4_CONT <= TONE3_SET;
			endcase
			
			CH4_S = LFSR[0];
			LFSR[14:0] = LFSR[15:1];
			if (NOISE_SET[2] == 0)
				LFSR[15] = LFSR[0];
			else
				LFSR[15] = !(LFSR[2]^LFSR[0]);
		end
		else
		begin
			FREQ4_CONT <= FREQ4_CONT - 1;
		end

		Sample_Out = 0;
		
		if (iSrc_Select == 0)
		begin
			if (CH1_S == 1) Sample_Out = VOL1_LOG>>3;
			if (CH2_S == 1) Sample_Out = Sample_Out + (VOL2_LOG>>3);
			if (CH3_S == 1) Sample_Out = Sample_Out + (VOL3_LOG>>3);
			if (CH4_S == 1) Sample_Out = Sample_Out + (VOL4_LOG>>3);
		end
	end

	//////////	16 Bits PISO MSB First	//////////////
	always@(negedge oAUD_BCK or negedge iRST_N)
	begin
		if(!iRST_N)
		SEL_Cont	<=	0;
		else
		SEL_Cont	<=	SEL_Cont+1;
	end

	assign	oAUD_DATA	=	Sample_Out[~SEL_Cont];												

endmodule
